/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "IDLValueType_Private.h"
#import "ObjectCell.h"
#import "IDLDefinition_Private.h"
#import "IDLValueClass.h"
#import "IDLInterface.h"
#import "IDLValueType_Private.h"
#import "IDLValueMember.h"
#import "IDLViewer.h"

@implementation IDLValueType(Viewer)

- (NSString*) title
{
    return @"valuetype";
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"value13"];
}

- (void) loadCell:cell
{
    [_support loadCell:cell];
    [cell setRepresentedObject:self];
	[cell setImage:[self cellImage]];
}

- (void) fillMatrix:matrix
{
    [_support fillMatrix:matrix];
}


- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard
{
    if([self isAbstract])
        return NO;
    else
        return [super writeDefinitionToPboard:pboard];
}

- (NSString*) category
{
    return @"- Values -";
}

- (NSString*)stringValue
{
    if(_name){
        NSString* modifier = @"";
        NSString* string = [_support stringValue];
        if(_modifier == vt_modifier_abstract)
            modifier = @"abstract ";
        else
        if(_modifier == vt_modifier_custom)
            modifier = @"custom ";
        return [NSString stringWithFormat:@"%@valuetype %@ %@", modifier, _name, string?string:@""];
    }else
        return @"ValueBase";
}

@end

@implementation IDLValueClass(Viewer)

- (NSString*) category
{
    return @"- Classes -";
}

- (NSString*) description
{
    return @"ValueClass";
}

- (NSString*) title
{
    return nil;
}

- (NSString*)stringValue
{
    return nil;
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"value13"];
}
- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:([_inheritance count] + [_supportArray count] + [_definitionDict count] + [_members count] == 0
        && _interface == nil && _super == nil)];
    [cell setRepresentedObject:self];
}

- (void) _addTitle:(NSString*)title toMatrix:matrix
{
    id cell;
    int n = [matrix numberOfRows];
    [matrix addRow];
    cell = [matrix cellAtRow:n column:0];
    [cell setStringValue:title];
    [cell setLeaf:YES];
    [cell setFont:[[NSFontManager sharedFontManager]convertFont:[cell font]
                        toHaveTrait:NSBoldFontMask]];
}

- (void) fillMatrix:matrix
{
    if(_super){
        id cell;
        int n = [matrix numberOfRows];
        [matrix addRow];
        cell = [matrix cellAtRow:n++ column:0];
        [cell setStringValue:[_super name]];
        [_super loadCell:cell];
    }
    [_inheritance fillMatrix:matrix];

    if(_interface || [_supportArray count] > 0){
        id cell;
        int n;

        [self _addTitle:@"- Supports -" toMatrix:matrix];
        
        n = [matrix numberOfRows];
        if(_interface){
            [matrix addRow];
            cell = [matrix cellAtRow:n++ column:0];
            [cell setStringValue:[_interface name]];
            [_interface loadCell:cell];
        }
        [_supportArray fillMatrix:matrix];
    }
    
    if([_definitionDict count] > 0){
        [self _addTitle:@"- Exports -" toMatrix:matrix];
        [_definitionDict fillMatrix:matrix];
    }
    
    if([_members count] > 0){
        [self _addTitle:@"- Members -" toMatrix:matrix];
        [_members fillMatrix:matrix];
    }
}

@end

@implementation IDLValueMember(Viewer)

- (void) fillMatrix:matrix
{
    [_type fillMatrix:matrix];
}

- (void) loadCell:cell
{
    [_type loadCell:cell];
    [cell setRepresentedObject:self];
}

- (NSString*) category
{
    return @"- Members -";
}

- (NSString*) title
{
    return [_type title];
}

- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard
{
    return NO;
}

- (NSString*) stringValue
{
    NSString* access;
    switch(_accessMode){
        default:
            access = @"";
            break;
        case vt_access_public:
            access = @"public ";
            break;
        case vt_access_private:
            access = @"private";
            break;
    }
    return [NSString stringWithFormat:@"%@ %@ %@",access,[_type stringValue],_name];
}

@end
