/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "IDLLog.h"
#import <AppKit/AppKit.h>

@protocol IDLLog
- (oneway void) postMessage;
@end

@implementation IDLLog

extern int dup2(int fd, int fd2);

- (oneway void) postMessage
{
    BOOL isSelected = [logView selectedRange].length > 0;
	NSData* data;
	NSMutableData* all = [NSMutableData data];
	[_lock lock];
	NSEnumerator* en = [_messages objectEnumerator];
	while(data = [en nextObject])
		[all appendData:data];
	[_messages removeAllObjects];
	[_lock unlock];
	NSString* message = [[[NSString alloc]initWithData:all encoding:NSUTF8StringEncoding]autorelease];
	[logView replaceCharactersInRange:NSMakeRange([[logView string] length],0) withString:message];
    if(!isSelected)
        [logView scrollRangeToVisible:NSMakeRange([[logView string]length], 0)];
    [window orderFront:self];
}

- (void) processLog:(NSArray*)portArray
{
	NSAutoreleasePool* rootPool = [[NSAutoreleasePool alloc]init];
	NSConnection* connection = [NSConnection connectionWithReceivePort:[portArray objectAtIndex:0]
				sendPort:[portArray objectAtIndex:1]];
	NSDistantObject* proxy = [connection rootProxy];
	[proxy setProtocolForProxy:@protocol(IDLLog)];
	
    while(1){
        NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
        NSData* data = [[_pipe fileHandleForReading]availableData];
        if([data length] > 0){
			[_lock lock];
			BOOL needsNotify = [_messages count] == 0;
			[_messages addObject:data];
			[_lock unlock];
			if(needsNotify)
				[(id<IDLLog>)proxy postMessage];
        }
        [pool release];
    }
	[rootPool release];
}

- init
{
    NSBundle* bundle = [NSBundle bundleForClass:[self class]];
    NSFileHandle* rfh, *wfh;
    [super init];
    if(![bundle loadNibFile:@"IDLLog.nib" 
                externalNameTable:[NSDictionary dictionaryWithObject:self forKey:@"NSOwner"]
                withZone:NULL]){
        NSLog(@"Cannot load IDLLog.nib");
        [self release];
        return nil;
    }
    [window setFrameUsingName:@"IDLLog"];
    [window setFrameAutosaveName:@"IDLLog"];
    [window setBecomesKeyOnlyIfNeeded:YES];
    [window setWorksWhenModal:YES];

	_messages = [[NSMutableArray alloc]init];
	_lock = [[NSLock alloc]init];
	NSPort* sendPort = [NSPort port];
	NSPort* recvPort = [NSPort port];
	_connection = [[NSConnection alloc]initWithReceivePort:recvPort sendPort:sendPort];
	[_connection setRootObject:self];

    _pipe = [[NSPipe allocWithZone:[self zone]]init];
    rfh = [_pipe fileHandleForReading];
    wfh = [_pipe fileHandleForWriting];
    dup2([wfh fileDescriptor],2);
    
    [NSThread detachNewThreadSelector:@selector(processLog:) toTarget:self 
			withObject:[NSArray arrayWithObjects:sendPort, recvPort, nil]];
    
    return self;
}

- (void) clearLog:sender
{
    [logView setString:@""];
}

- (void) orderFront:sender
{
    [window orderFront:self];
}

@end
