/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "IDLRunController.h"
#import "IDLDefinition_Private.h"
#import "IDLItem.h"
#import "IDLFunction.h"
#import "IDLAttribute.h"
#import "IDLInterface_Private.h"
#import "CORBAObject.h"
#import "IDLExceptionController.h"
#import "IDLException.h"
#import "IDLViewer.h"
#import "ResponseSimulator.h"

@implementation IDLRunController

static NSPoint _topLeftPoint = {0,0};
static BOOL _validOffset = NO;
static NSLock* _lock = nil;

extern NSString *CORBAIORPboardType;

+ (void) initialize
{
	if([IDLRunController class] == self){
		_lock = [[NSLock alloc]init];
/*
		[self setKeys:[NSArray arrayWithObjects:@"selectedItem", @"selectedItemValue", nil]
				triggerChangeNotificationsForDependentKey:@"insertAllowed"];
		[self setKeys:[NSArray arrayWithObjects:@"selectedItem", @"selectedItemValue", nil]
				triggerChangeNotificationsForDependentKey:@"removeAllowed"];
		[self setKeys:[NSArray arrayWithObjects:@"connection", @"usesSimulator", nil]
				triggerChangeNotificationsForDependentKey:@"canSend"];
*/
	}
}

+ (NSSet*) keyPathsForValuesAffectingInserAllowed
{
	return [NSSet setWithObjects:@"selectedItem", @"selectedItemValue", nil];
}
+ (NSSet*) keyPathsForValuesAffectingRemoveAllowed
{
	return [NSSet setWithObjects:@"selectedItem", @"selectedItemValue", nil];
}
+ (NSSet*) keyPathsForValuesAffectingCanSend
{
	return [NSSet setWithObjects:@"connection", @"useSimulator", nil];
}

- (void) _offsetWindow
{
	if(_validOffset){
		_topLeftPoint = [window cascadeTopLeftFromPoint:_topLeftPoint];
		[window setFrameTopLeftPoint:_topLeftPoint];
	}else{
		NSRect frame = [window frame];
		_topLeftPoint.x = NSMinX(frame);
		_topLeftPoint.y = NSMaxY(frame);
		_validOffset = YES;
	}
}

- (NSString*) nibName
{
	return @"IDLRunController";
}

- (BOOL) isEditable
{
	return YES;
}

- initWithDefinition:(IDLDefinition*)def
{
	self = [super init];
	_item = [def createIDLItemWithName:[def name]];
	[_item setEditable:[self isEditable]];
	[_item setController:self];
	[NSBundle loadNibNamed:[self nibName] owner:self];
	[self _offsetWindow];
	[[scrollView documentView]reloadData];
	id connection = [[_item interfaceObject]interfaceConnection];
	if(connection)
		[self setConnection:connection];
	[[NSNotificationCenter defaultCenter]addObserver:self
                selector:@selector(invalidateConnection:)
                name:ORBInvalidateObject
                object:nil];
	[[NSNotificationCenter defaultCenter]addObserver:self
                selector:@selector(_simulatorDidActivate:)
                name:SimulatorDidActivate
                object:nil];
	[[NSNotificationCenter defaultCenter]addObserver:self
                selector:@selector(_simulatorWillDeactivate:)
                name:SimulatorWillDeactivate
                object:nil];
	if([ResponseSimulator simulator])
		[self setSimulatorAvailable:YES];
	return self;
}

// Opens runner that shares item's value
// If value is mutable the value changes will be reflected 
// in both original 'item' and new item until the value is replaced
- initWithIDLItem:(IDLItem*)item
{
	self = [super init];
	_item = [[item itemType] createIDLItemWithName:[item itemName]];
	[_item setEditable:[item isEditable]];
	[_item setController:self];
	[_item setValue:[item value]];
	[NSBundle loadNibNamed:[self nibName] owner:self];
	[self _offsetWindow];
	[[scrollView documentView]reloadData];
	id connection = [[_item interfaceObject]interfaceConnection];
	if(connection)
		[self setConnection:connection];
	[[NSNotificationCenter defaultCenter]addObserver:self
                selector:@selector(invalidateConnection:)
                name:ORBInvalidateObject
                object:nil];
	[[NSNotificationCenter defaultCenter]addObserver:self
                selector:@selector(_simulatorDidActivate:)
                name:SimulatorDidActivate
                object:nil];
	[[NSNotificationCenter defaultCenter]addObserver:self
                selector:@selector(_simulatorWillDeactivate:)
                name:SimulatorWillDeactivate
                object:nil];
	if([ResponseSimulator simulator])
		[self setSimulatorAvailable:YES];
	return self;
}

- (void) awakeFromNib
{
	[[scrollView documentView]setAutoresizesOutlineColumn:NO];
	[[scrollView documentView] registerForDraggedTypes:[NSArray arrayWithObjects:
            IDLDefinitionPboardType, IDLItemPboardType, CORBAIORPboardType, NSStringPboardType, nil]];
}

- (void) dealloc
{
	[[NSNotificationCenter defaultCenter]removeObserver:self];
	[_connection release];
	[_status release];
	[_item release];
	[super dealloc];
}

- connection
{
	[_lock lock];
	id connection = [_connection retain];
	[_lock unlock];
	return [connection autorelease];
}

- (void) setConnection:conn
{
	[_lock lock];
	[_connection release];
	_connection = [conn retain];
    [_lock unlock];
}

- (void) invalidateConnection:(NSNotification*)notification
{
    if([notification object] == _connection)
		[self setConnection:nil];
}

- (BOOL) acceptConnections:(NSArray*)array
{
	IDLInterface* ifObj = [_item interfaceObject];
    if(ifObj){
        NSEnumerator* en = [array objectEnumerator];
        id obj;
        while(obj = [en nextObject]){
            if(![obj interface])
                [obj narrowInterface];
            if([[obj interface]adoptsInterface:ifObj]){// == or inherited from ifObj
				[self setConnection:obj];
                return YES;
            }
        }
    }
    return NO;
}

- window
{
	return window;
}

- (BOOL) isSending
{
	return _flags.sending;
}

- (void) setSending:(BOOL)flag
{
	[self willChangeValueForKey:@"sending"];
	_flags.sending = flag?YES:NO;
	[self didChangeValueForKey:@"sending"];
}

- (BOOL) isAttribute
{
	return [[_item itemType]isKindOfClass:[IDLAttribute class]];
}

- (BOOL) isFunction
{
	return [[_item itemType]isKindOfClass:[IDLFunction class]];
}

// Get for attribute or send for function
- (BOOL) isSendable
{
	return [self isFunction] || [self isAttribute];
}

- (IDLItem*) item
{
	return _item;
}

- (void) presentException:(NSException*)exception
{
    NSDictionary* info = [exception userInfo];
    NSString* typeId = [info objectForKey:IDLExceptionIDKey];
    IDLException* excp = nil;
    if(typeId){
        IDLDefinitionScope* scope = [[_item interfaceObject]rootScope];
        excp = (IDLException*)[scope definitionForTypeId:typeId];
    }
    IDLExceptionController* ctrl = [[IDLExceptionController alloc]initWithException:exception definition:excp];
    [ctrl showExceptionFor:self window:window];
    [self setValue:[exception name] forKey:@"status"];
}

- (void) _updateItemWithValue:(NSDictionary*)values
{
    if(_flags.sending){
        NSException* exception = [values objectForKey:@"$exception"];
        [_lock lock];
        if(exception){
            [self presentException:exception];
        }else{
            @try{
				NSNumber* duration = [values objectForKey:@"$duration"];
				NSString* attr = [values objectForKey:@"$attribute"];
				if([attr isEqualToString:@"get"]){ // get attribute
					[_item setItemValue:[values objectForKey:@"$return"]];
					[[scrollView documentView]reloadItem:_item reloadChildren:YES];
				}else
				if(!attr){// regular function
					[_item setValue:values forKey:@"itemValue"];
					[[scrollView documentView]reloadItem:_item reloadChildren:YES];
				}
				[self setValue:[NSString stringWithFormat:@"Finished, duration %6.3f sec.",
					duration?[duration doubleValue]:0.0] forKey:@"status"];

            }@catch(NSException *localException){
				//[self presentException:localException];
            }
        }
		[self setSending:NO];
        [_lock unlock];
    }
}

- (void) _sendFunctionWithArguments:(NSDictionary*)args
{
    NSMutableDictionary* values = nil;
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    NSDate* date = [NSDate date];
    IDLFunction* fo = [args objectForKey:@"$function"];
	NSString* attr = [args objectForKey:@"$attribute"]; // @"get", @"set" or nil
    @try{
		[[[_item interfaceObject]rootScope]set];
		values = [_connection invokeFunction:fo with:args];
		[values setObject:[NSNumber numberWithDouble:[[NSDate date]timeIntervalSinceDate:date]]
				forKey:@"$duration"];
		if(attr)
			[values setObject:attr forKey:@"$attribute"];
	}@catch(NSException *localException){
		values = [NSDictionary dictionaryWithObject:localException forKey:@"$exception"];
	}
    [self performSelectorOnMainThread:@selector(_updateItemWithValue:)
        withObject:values waitUntilDone:NO];
    [pool release];
}

- (void) simulateSendFunction
{
	ResponseSimulator* simulator = [ResponseSimulator simulator];
    if(simulator && [window makeFirstResponder:window]){
		NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
		NSDate* date = [NSDate date];
		IDLFunction* fo = (IDLFunction*)[_item itemType];
		NSMutableDictionary* values = nil;
		NSMutableDictionary* args = [NSMutableDictionary dictionaryWithDictionary:[_item value]];
		[args setObject:fo forKey:@"$function"];
        [self setSending:YES];
		@try{
			[[[_item interfaceObject]rootScope]set];
			values = (NSMutableDictionary*)[fo simulateInvokeWith:args response:[simulator response]];
		}@catch(NSException *localException){
			values = [NSMutableDictionary dictionaryWithObject:localException forKey:@"$exception"];
		}
		NSTimeInterval t = [[NSDate date]timeIntervalSinceDate:date];
		[values setObject:[NSNumber numberWithDouble:t]forKey:@"$duration"];
		[self _updateItemWithValue:values];
		[pool release];
	}
}

- (void) simulateFetchAttribute
{
	ResponseSimulator* simulator = [ResponseSimulator simulator];
    if(simulator && [window makeFirstResponder:window]){
		NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
        IDLFunction* fo = [(IDLAttribute*)[_item itemType]getAttribute];
		NSDictionary* values = nil;
        NSMutableDictionary* args = [NSMutableDictionary dictionaryWithObjectsAndKeys:
            fo, @"$function", @"get", @"$attribute", nil];
        [self setSending:YES];
		@try{
			[[[_item interfaceObject]rootScope]set];
			values = [fo simulateInvokeWith:args response:[simulator response]];
		}@catch(NSException *localException){
			values = [NSMutableDictionary dictionaryWithObject:localException forKey:@"$exception"];
		}
		[self _updateItemWithValue:values];
		[pool release];
    }
}

- (void) sendFunction
{
    if([window makeFirstResponder:window]){
        NSMutableDictionary* args = [NSMutableDictionary dictionaryWithDictionary:[_item value]];
        [args setObject:[_item itemType] forKey:@"$function"];
        [self setValue:@"Sending ..." forKey:@"status"];
        [_lock lock];
        [self setSending:YES];
        [NSThread detachNewThreadSelector:@selector(_sendFunctionWithArguments:)
            toTarget:self withObject:args];
        [_lock unlock];
    }
}

- (void) sendAttribute
{
    if([window makeFirstResponder:window]){
        IDLFunction* fo = [(IDLAttribute*)[_item itemType]setAttribute];
        NSDictionary* args = [NSDictionary dictionaryWithObjectsAndKeys:
            [_item value], [_item itemName],
            fo, @"$function",
            @"set", @"$attribute", nil];
        [self setValue:@"Sending ..." forKey:@"status"];
        [_lock lock];
        [self setSending:YES];
        [NSThread detachNewThreadSelector:@selector(_sendFunctionWithArguments:)
            toTarget:self withObject:args];
        [_lock unlock];
    }
}

- (void) fetchAttribute
{
    if([window makeFirstResponder:window]){
        IDLFunction* fo = [(IDLAttribute*)[_item itemType]getAttribute];
        NSDictionary* args = [NSDictionary dictionaryWithObjectsAndKeys:
            fo, @"$function", @"get", @"$attribute", nil];
        [self setValue:@"Sending ..." forKey:@"status"];
        [_lock lock];
        [self setSending:YES];
        [NSThread detachNewThreadSelector:@selector(_sendFunctionWithArguments:)
            toTarget:self withObject:args];
        [_lock unlock];
    }
}

- (void) sendOrGet
{
	if([[_item itemType]isKindOfClass:[IDLFunction class]]){
		if([self usesSimulator])
			[self simulateSendFunction];
		else
			[self sendFunction];
	}else
	if([[_item itemType]isKindOfClass:[IDLAttribute class]]){
		if([self usesSimulator])
			[self simulateFetchAttribute];
		else
			[self fetchAttribute];
	}
}

- selectedItemValue
{
	NSOutlineView* table = [scrollView documentView];
	int row = [table selectedRow];
	if(row >= 0)
		return [[table itemAtRow:row]value];
	else
		return nil;
}

- (void) invalidateItem:(IDLItem*)item
{
	if(item && [item isInTree])
		[[scrollView documentView]reloadItem:item reloadChildren:YES];
}

- (void) expandItem:(IDLItem*)item
{
	[[scrollView documentView]expandItem:item];
}
- (void) collapseItem:(IDLItem*)item
{
	[[scrollView documentView]collapseItem:item];
}

- (void) clearItemValue:sender
{
	NSOutlineView* table = [scrollView documentView];
	int row = [table selectedRow];
	if(row >= 0){
		IDLItem* item = [table itemAtRow:row];
		if([item isEditable] && [item respondsToSelector:@selector(clearItemValue)]){
			[self willChangeValueForKey:@"selectedItem"];
			[table collapseItem:item];
			[(IDLValueItem*)item clearItemValue];
			[self didChangeValueForKey:@"selectedItem"];
		}
	}
}
- (void) setItemValue:sender
{
	NSOutlineView* table = [scrollView documentView];
	int row = [table selectedRow];
	if(row >= 0){
		IDLItem* item = [table itemAtRow:row];
		if([item respondsToSelector:@selector(setItemValue)]){
			[self willChangeValueForKey:@"selectedItem"];
			[table collapseItem:item];
			[(IDLValueItem*)item setItemValue];
			[self didChangeValueForKey:@"selectedItem"];
		}
	}
}

- (void) resetItemValue:sender
{
	NSOutlineView* table = [scrollView documentView];
	int row = [table selectedRow];
	if(row >= 0){
		IDLItem* item = [table itemAtRow:row];
		if([item respondsToSelector:@selector(resetItemValue)]){
			[self willChangeValueForKey:@"selectedItem"];
			[table collapseItem:item];
			[(IDLValueItem*)item resetItemValue];
			[self didChangeValueForKey:@"selectedItem"];
		}
	}
}

- (void) windowWillClose:(NSNotification*)note
{
	[self autorelease];
}


- (id)outlineView:(NSOutlineView*)outlineView child:(int)index ofItem:(id)item
{
	if(item)
		return [[item items]objectAtIndex:index];
	else
		return _item;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	return ![item isLeaf];
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	if(item)
		return [item itemCount];
	else
		return _item?1:0;
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	NSString* identifier = [tableColumn identifier];
	return [item valueForKey:identifier];
}

- (void)outlineView:(NSOutlineView *)outlineView setObjectValue:(id)object 
		forTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	NSString* identifier = [tableColumn identifier];
	[item setValue:object forKey:identifier];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldEditTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	if([tableColumn isEditable])
		return [item isItemEditable];
	return NO;
}

- (void) outlineView:(NSOutlineView*)outlineView willDisplayCell:(id)cell 
		forTableColumn:(NSTableColumn *)tableColumn item:(IDLItem*)item
{
//	[cell setFormatter:[item formatter]];
}

- (NSString*) outlineView:(NSOutlineView*)view toolTipForCell:(NSCell*)cell rect:(NSRectPointer)rect 
		tableColumn:(NSTableColumn*)tc item:(id)item mouseLocation:(NSPoint)mouseLocation
{
	if([[tc identifier]isEqualToString:@"itemName"])
		return [[item itemType]stringValue];
	else
		return [item valueDescription];
}
- (BOOL)control:(NSControl *)control didFailToFormatString:(NSString *)string errorDescription:(NSString *)error
{
	NSBeep();
	_flags.editInvalid = YES;
	return YES;
}

- (BOOL)control:(NSControl *)control textShouldEndEditing:(NSText *)fieldEditor
{
	if(_flags.editInvalid){
		[fieldEditor selectAll:self];
		_flags.editInvalid = NO;
		return NO;
	}else
		return YES;
}

- (NSArray*) pasteboardTypesForDragMask:(NSDragOperation)mask
{
	NSMutableArray* types = [NSMutableArray arrayWithObjects:IDLItemLinkPboardType, IDLItemPboardType, IDLDefinitionPboardType, CORBAIORPboardType, NSStringPboardType, nil];
	if(!(mask & NSDragOperationLink))
		[types removeObject:IDLItemLinkPboardType];
	if(!(mask & NSDragOperationCopy))
		[types removeObject:IDLItemPboardType];
	return types;
}

- (NSDragOperation)outlineView:(NSOutlineView *)outlineView 
		validateDrop:(id <NSDraggingInfo>)info 
		proposedItem:(IDLItem*)item
		proposedChildIndex:(int)index
{
	if(item && ![self isSending] && [item isEditable]){
		NSPasteboard* pboard = [info draggingPasteboard];
		id source = [info draggingSource];
		NSDragOperation sourceDragMask = [info draggingSourceOperationMask];
		NSArray* types = [self pasteboardTypesForDragMask:sourceDragMask];		
		NSString* type = [pboard availableTypeFromArray:types];
		if([type isEqualToString:IDLItemLinkPboardType]){
			NSData* data = [pboard dataForType:type];
			IDLItem* newItem = nil;
			[data getBytes:&newItem length:sizeof(newItem)];
			if(index == NSOutlineViewDropOnItemIndex){
				if([item shouldTakeValueFromItem:newItem])
					return NSDragOperationLink & sourceDragMask;
			}else{
				if([item shouldInsertItem:newItem])
					return NSDragOperationLink & sourceDragMask;
			}
		}else
		if([type isEqualToString:IDLItemPboardType]){
			NSData* data = [pboard dataForType:type];
			IDLItem* newItem = [NSKeyedUnarchiver unarchiveObjectWithData:data];
			if(index == NSOutlineViewDropOnItemIndex){
				if([item shouldTakeValueFromItem:newItem])
					return NSDragOperationCopy & sourceDragMask;
			}else{
				if([item shouldInsertItem:newItem])
					return NSDragOperationCopy & sourceDragMask;
			}
		}else
		if([type isEqualToString:IDLDefinitionPboardType]){
			IDLDefinition* def = [source representedObject];
			if(def){
				if(index == NSOutlineViewDropOnItemIndex){
					if([item shouldSetIDLType:def])
						return NSDragOperationGeneric & sourceDragMask;
				}else{
					if([item shouldInsertItemWithIDLType:def])
						return NSDragOperationGeneric & sourceDragMask;
				}
			}
		}else
		if([type isEqualToString:NSStringPboardType] || [type isEqualToString:CORBAIORPboardType]){
			if(index == NSOutlineViewDropOnItemIndex && [item isEditable] &&
					([item isKindOfClass:[IDLIORItem class]] || [item isKindOfClass:[IDLAbstractItem class]]))
				return NSDragOperationGeneric & sourceDragMask;
		}
	}
	return NSDragOperationNone;
}

- (void) endEditing
{
	if([[scrollView documentView]currentEditor])
		[[scrollView documentView]abortEditing];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView 
		acceptDrop:(id <NSDraggingInfo>)info 
		item:(IDLItem*)item
		childIndex:(int)index
{
	if(item){
		NSPasteboard* pboard = [info draggingPasteboard];
		id source = [info draggingSource];
		NSDragOperation sourceDragMask = [info draggingSourceOperationMask];
		NSArray* types = [self pasteboardTypesForDragMask:sourceDragMask];		
		NSString* type = [pboard availableTypeFromArray:types];
		if([type isEqualToString:IDLItemLinkPboardType]){
			NSData* data = [pboard dataForType:type];
			IDLItem* newItem = nil;
			[data getBytes:&newItem length:sizeof(newItem)];
			if(index == NSOutlineViewDropOnItemIndex){
				if([item shouldTakeValueFromItem:newItem]){
					[self endEditing];
					[item takeValueFromItem:newItem];
					[[scrollView documentView]reloadItem:item reloadChildren:YES];
					return YES;
				}
			}else{
				if([item shouldInsertItem:newItem]){
					//[self endEditing];
					// cannot share newItem. create a copy
					IDLItem* item1 = [IDLItem itemWithIDLType:[newItem itemType]];
					[item1 setValue:[newItem value]];
					[item insertItem:item1 atIndex:index];
					[[scrollView documentView]reloadItem:item reloadChildren:YES];
					return YES;
				}
			}
		}else
		if([type isEqualToString:IDLItemPboardType]){
			NSData* data = [pboard dataForType:type];
			IDLItem* newItem = [NSKeyedUnarchiver unarchiveObjectWithData:data];
			if(index == NSOutlineViewDropOnItemIndex){
				if([item shouldTakeValueFromItem:newItem]){
					[self endEditing];
					[item takeValueFromItem:newItem];
					[[scrollView documentView]reloadItem:item reloadChildren:YES];
					return YES;
				}
			}else{
				if([item shouldInsertItem:newItem]){
					//[self endEditing];
					[item insertItem:newItem atIndex:index];
					[[scrollView documentView]reloadItem:item reloadChildren:YES];
					return YES;
				}
			}
		}else
		if([type isEqualToString:IDLDefinitionPboardType]){
			IDLDefinition* def = [source representedObject];
			if(def){
				if(index == NSOutlineViewDropOnItemIndex){
					if([item shouldSetIDLType:def]){
						[self endEditing];
						[item setIDLType:def];
						[[scrollView documentView]reloadItem:item reloadChildren:YES];
						return YES;
					}
				}else{
					if([item shouldInsertItemWithIDLType:def]){
						//[self endEditing];
						[item insertNewItemWithIDLType:def atIndex:index];
						[[scrollView documentView]reloadItem:item reloadChildren:YES];
						return YES;
					}
				}
			}
		}else
		if([type isEqualToString:NSStringPboardType] || [type isEqualToString:CORBAIORPboardType]){
			if(index == NSOutlineViewDropOnItemIndex && [item isEditable] && 
					([item isKindOfClass:[IDLIORItem class]] || [item isKindOfClass:[IDLAbstractItem class]]))
				return [(IDLIORItem*)item takeValueFromPasteboard:pboard forType:type];
		}
	}
	return NO;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView writeItems:(NSArray *)array toPasteboard:(NSPasteboard *)pboard
{
	if([array count] > 0){
		IDLItem* item = [array objectAtIndex:0];
		return [item writeToPasteboard:pboard];
	}else
		return NO;
}

- (void) copy:sender
{
	int row = [[scrollView documentView]selectedRow];
	if(row >= 0){
		IDLItem* item = [[scrollView documentView]itemAtRow:row];
		NSData* data = [NSKeyedArchiver archivedDataWithRootObject:item];
		NSPasteboard* pboard = [NSPasteboard generalPasteboard];
		[pboard declareTypes:[NSArray arrayWithObject:IDLItemPboardType] owner:nil];
		[pboard setData:data forType:IDLItemPboardType];
	}
}

- (int) parentRowForRow:(int)row index:(int*)index
{
	int level = [[scrollView documentView]levelForRow:row];
	int n = 0;
	while(row-- > 0){
		int l = [[scrollView documentView]levelForRow:row];
		if(l < level)
			break;
		if(l == level)
			n ++;
	}
	if(index) *index = n;
	return row;
}

// Checks if new item can be inserted first as a child and then as a sibling
// of a selected item, inserts the new item  and selects it
// Otherwise checks if the value of the selected item can be replaced by
// the value of new item. Replaces the value if allowed. Selection remains unchanged.
- (void) paste:sender
{
	int row = [[scrollView documentView]selectedRow];
	if(row >= 0){
		IDLItem* item = [[scrollView documentView]itemAtRow:row];
		NSPasteboard* pboard = [NSPasteboard generalPasteboard];
		NSString* type = [pboard availableTypeFromArray:[NSArray arrayWithObject:IDLItemPboardType]];
		if(type){
			NSData* data = [pboard dataForType:type];
			IDLItem* newItem = [NSKeyedUnarchiver unarchiveObjectWithData:data];
			if([item shouldInsertItem:newItem]){
				[item insertItem:newItem atIndex:0];
				[[scrollView documentView]reloadItem:item reloadChildren:YES];
				[[scrollView documentView]expandItem:item];
				row = [[scrollView documentView]rowForItem:newItem];
				[[scrollView documentView]selectRow:row byExtendingSelection:NO];
				return;
			}else{
				int index = 0;
				int n = [self parentRowForRow:row index:&index];
				IDLItem* parent = (n >= 0)? [[scrollView documentView]itemAtRow:n] : nil;
				if(parent && [parent shouldInsertItem:newItem]){
					[parent insertItem:newItem atIndex:index+1];
					[[scrollView documentView]reloadItem:parent reloadChildren:YES];
					row = [[scrollView documentView]rowForItem:newItem];
					[[scrollView documentView]selectRow:row byExtendingSelection:NO];
					return;
				}else
				if([item shouldTakeValueFromItem:newItem]){
					[item takeValueFromItem:newItem];
					[[scrollView documentView]reloadItem:item reloadChildren:YES];
					return;
				}
			}
		}
	}
	NSBeep();
}

- (void) cut:sender
{
	[self copy:sender];
	[self removeSelectedItem:sender];
}

// Removes selected item if allowed. Selects previous sibling, if any, or parent
- (void) removeSelectedItem:sender
{
	int row = [[scrollView documentView]selectedRow];
	if(row >= 0){
		int index = 0;
		int n = [self parentRowForRow:row index:&index];
		IDLItem* parent = (n >= 0)? [[scrollView documentView]itemAtRow:n] : nil;
		if([parent respondsToSelector:@selector(removeItemAtIndex:)] && [parent isEditable]){
			[(IDLSequenceItem*)parent removeItemAtIndex:index];
			[[scrollView documentView]reloadItem:parent reloadChildren:YES];
			int level = [[scrollView documentView]levelForRow:n];
			while(row-- > n)
				if([[scrollView documentView]levelForRow:row] <= level+1)
					break;
			[[scrollView documentView]selectRow:row byExtendingSelection:NO];
			return;
		}
	}
	NSBeep();
}

// Inserts a new item if allowed, and selects it, if its parent is expanded
// The new item is created with the idl type of the parent sequence
// First tries to insert a child.
// If child is not allowed, tries to insert a sibling
- (void) insertNewItem:sender
{
	int row = [[scrollView documentView]selectedRow];
	if(row >= 0){
		IDLItem* item = [[scrollView documentView]itemAtRow:row];
		if([item respondsToSelector:@selector(insertNewItemAtIndex:)]){
			if([item isEditable]){
				IDLItem* newItem = [(IDLSequenceItem*)item insertNewItemAtIndex:0];
				[[scrollView documentView]reloadItem:item reloadChildren:YES];
				if(newItem){
					row = [[scrollView documentView]rowForItem:newItem];
					[[scrollView documentView ]selectRow:row byExtendingSelection:NO];
				}
			}else
				NSBeep();
		}else{
			int index = 0;
			int n = [self parentRowForRow:row index:&index];
			IDLItem* parent = (n >= 0)? [[scrollView documentView]itemAtRow:n] : nil;
			if([parent respondsToSelector:@selector(insertNewItemAtIndex:)]){
				if([parent isEditable]){
					IDLItem* newItem = [(IDLSequenceItem*)parent insertNewItemAtIndex:index+1];
					[[scrollView documentView]reloadItem:parent reloadChildren:YES];
					if(newItem){
						row = [[scrollView documentView]rowForItem:newItem];
						[[scrollView documentView ]selectRow:row byExtendingSelection:NO];
					}
				}else
					NSBeep();
			}
		}
	}else
		NSBeep();	
}

// KVC method. Returns if selected item can be removed
- (BOOL) isRemoveAllowed
{
	int row = [[scrollView documentView]selectedRow];
	int n = [self parentRowForRow:row index:NULL];
	IDLItem* parent = (n >= 0)? [[scrollView documentView]itemAtRow:n] : nil;
	return [parent respondsToSelector:@selector(shouldRemoveItem)] && [(IDLSequenceItem*)parent shouldRemoveItem];
}

// KVC method. Returns if a new item can be inserted as a child or a sibling of the selected item
- (BOOL) isInsertAllowed
{
	int row = [[scrollView documentView]selectedRow];
	if(row >= 0){
		IDLItem* item = [[scrollView documentView]itemAtRow:row];
		if([item respondsToSelector:@selector(shouldInsertNewItem)])
			return [(IDLSequenceItem*)item shouldInsertNewItem];
		else{
			int n = [self parentRowForRow:row index:NULL];
			IDLItem* parent = (n >= 0)? [[scrollView documentView]itemAtRow:n] : nil;
			if([parent respondsToSelector:@selector(shouldInsertNewItem)])
				return [(IDLSequenceItem*)parent shouldInsertNewItem];
		}
	}
	return NO;
}

- (void) setUsesSimulator:(BOOL)flag
{
	_flags.usesSimulator = flag?YES:NO;
}

- (BOOL) usesSimulator
{
	return _flags.usesSimulator;
}

- (BOOL) isSimulatorAvailable
{
	return _flags.simulatorAvailable;
}

- (void) setSimulatorAvailable:(BOOL)flag
{
	_flags.simulatorAvailable = flag?YES:NO;
}

- (BOOL) canSend
{
	return _connection != nil || [self usesSimulator];
}

// Triggers KVO notiications for 'removeAllowed' and 'insertAllowed' keys
- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
	[self willChangeValueForKey:@"selectedItem"];
	[self didChangeValueForKey:@"selectedItem"];
}

- (IDLDefinitionScope*)rootScope
{
	return _rootScope;
}

- (void) setRootScope:(IDLDefinitionScope*)scope
{
	if(scope != _rootScope){
		[_rootScope release];
		_rootScope = [scope retain];
	}
}

- (void) setScope
{
	[_rootScope set];
}

- (void) openIDLRunner:sender
{
	int row = [[scrollView documentView]selectedRow];
	if(row >= 0){
		IDLItem* item = [[scrollView documentView]itemAtRow:row];
		IDLRunController* runner = [[IDLRunController alloc]initWithIDLItem:item];
		[[runner window]makeKeyAndOrderFront:self];
	}else
		NSBeep();
}

- (BOOL)validateMenuItem:menuItem
{
	SEL action = [menuItem action];
	if(action == @selector(copy:)){
		int row = [[scrollView documentView]selectedRow];
		return row >= 0;
	}else
	if(action == @selector(paste:)){
		int row = [[scrollView documentView]selectedRow];
		if(row >= 0){
			IDLItem* item = [[scrollView documentView]itemAtRow:row];
			NSPasteboard* pboard = [NSPasteboard generalPasteboard];
			NSString* type = [pboard availableTypeFromArray:[NSArray arrayWithObject:IDLItemPboardType]];
			if(type){
				NSData* data = [pboard dataForType:type];
				IDLItem* newItem = [NSKeyedUnarchiver unarchiveObjectWithData:data];
				if([item shouldInsertItem:newItem])
					return YES;
				else{
					int n = [self parentRowForRow:row index:NULL];
					IDLItem* parent = (n >= 0)? [[scrollView documentView]itemAtRow:n] : nil;
					if(parent && [parent shouldInsertItem:newItem])
						return YES;
					else
						return [item shouldTakeValueFromItem:newItem];
				}
			}
		}
		return NO;
	}else
	if(action == @selector(removeSelectedItem:) || action == @selector(cut:)){
		return [self isRemoveAllowed];
	}else
	if(action == @selector(insertNewItem:)){
		return [self isInsertAllowed];
	}else
	if(action == @selector(clearItemValue:)){
		int row = [[scrollView documentView]selectedRow];
		if(row >= 0){
			IDLItem* item = [[scrollView documentView]itemAtRow:row];
			return [item isEditable] && [item respondsToSelector:@selector(clearItemValue)] && [item itemValue] != nil;
		}
		return NO;
	}else
	if(action == @selector(resetItemValue:)){
		int row = [[scrollView documentView]selectedRow];
		if(row >= 0){
			IDLItem* item = [[scrollView documentView]itemAtRow:row];
			return [item isEditable] && [item respondsToSelector:@selector(resetItemValue)];
		}
		return NO;
	}else
	if(action == @selector(openIDLRunner:)){
		int row = [[scrollView documentView]selectedRow];
		return row >= 0;
	}
	return YES;
}

- (void) _simulatorDidActivate:(NSNotification*)note
{
	[self setSimulatorAvailable:YES];
}

- (void) _simulatorWillDeactivate:(NSNotification*)note
{
	[self setUsesSimulator:NO];
	[self setSimulatorAvailable:NO];
}

@end
