/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>

@class IDLDefinition, IDLValueType, VXConnection;

@protocol IDLInvocationObject
- (VXConnection*) connection;
@end

@protocol CORBACoder
- (id<IDLInvocationObject>)invocationObject;

- (void) encodeChar:(unsigned char)value;
- (void) encodeShort:(short)value;
- (void) encodeUShort:(unsigned short)value;
- (void) encodeLong:(long)value;
- (void) encodeULong:(unsigned long)value;
- (void) encodeBOOL:(BOOL)value;
- (void) encodeFloat:(float)value;
- (void) encodeDouble:(double)value;
- (void) encodeLongLong:(long long)value;
- (void) encodeULongLong:(unsigned long long)value;
- (void) encodeString:(NSString*)string;
- (void) encodeObject:object;
- (void) encodeStruct:object usingObjectCoder:coder;
- (void) encodeSequence:object usingObjectCoder:coder;
- (void) encodeData:stream;
- (void) encodeData:stream length:(unsigned)length;
- (void) encodeIndirection:(long)location;
- (long) location;
- (void) encodeUniChar:(unichar)value;
- (void) encodeWString:(NSString*)value;
- (long) locationForDefinition:(IDLDefinition*)def;
- (void) encodeValue:obj forObjectCoder:(IDLValueType*)valuetype;
@end

@protocol CORBADecoder
- (id<IDLInvocationObject>)invocationObject;

- (void) decodeChar:(unsigned char*)value;
- (void) decodeShort:(short*)value;
- (void) decodeUShort:(unsigned short*)value;
- (void) decodeLong:(long*)value;
- (void) decodeULong:(unsigned long*)value;
- (void) decodeBOOL:(BOOL*)value;
- (void) decodeFloat:(float*)value;
- (void) decodeDouble:(double*)value;
- (void) decodeLongLong:(long long*)value;
- (void) decodeULongLong:(unsigned long long*)value;
- (void) decodeString:(NSString**)value;
- (void) decodeObject:(id*)object;
- (void) decodeStruct:(id*)object usingObjectCoder:coder;
- (void) decodeSequence:(id*)object usingObjectCoder:coder;
- (void) decodeData:(id*)stream;
- (void) decodeData:(id*)stream length:(unsigned)length;
- (IDLDefinition*) definitionWithLocation:(long)location;
- (void) registerDefinition:(IDLDefinition*)def forLocation:(long)location;
- (long) location;
- (unsigned) cursorPosition;
- (long) length;
- (void) decodeUniChar:(unichar*)value;
- (void) decodeWString:(NSString**)value;

- (void) decodeValue:(id*)obj forObjectCoder:(IDLValueType*)valuetype;

@end

@protocol CORBACoding

- (void) encodeObject:obj withCoder:(id <CORBACoder>)coder;
- (void) decodeObject:(id*)obj withCoder:(id <CORBADecoder>)coder;
//- (void) encodeObject:object forCoder:(id <CORBACoder>)coder;
//- (void) decodeObject:(id *)object forCoder:(id <CORBADecoder>)coder;
- (void) encodeWithCORBACoder:(id <CORBACoder>)coder;

@end

extern NSString* CORBAMarshalException;
extern NSString* CORBAInvalidArgumentException;

