/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLAny.h"
#import "IDLStruct.h"
#import "IDLUnion.h"
#import "IDLArray.h"
#import "IDLEnum.h"
#import "IDLSequence.h"
#import "IDLTypedef.h"
#import "IDLException.h"
#import "IDLInterface_Private.h"
#import "IDLTypedef.h"
#import "IDLNative.h"
#import "IDLDefinitionScope_Private.h"
#import "IDLEncapsulation.h"
#import "IDLValueType_Private.h"

@implementation IDLDefinition(CORBACoding)

NSString* CORBAMarshalException = @"CORBAMarshalException";

- (void) encodeObject:obj withCoder:(id <CORBACoder>)coder
{
}

- (void) decodeObject:(id*)obj withCoder:(id <CORBADecoder>)coder
{
    *obj = nil;
}

- (void) encodeWithCORBACoder:(id<CORBACoder>)coder
{
    [coder encodeLong:[self kind]];
}

- (const char*) objCType
{
    return NULL;
}
- (unsigned) argLength
{
    return 0;
}
- (unsigned) returnValueLength
{
    return 0;
}

- argumentObjectFromInvocation:(NSInvocation*)inv atIndex:(int)n
{
    return nil;
}
- argumentObjectOfType:(const char*)argType fromPointer:(void*)ptr
{
    return nil;
}

- (void) setValue:value ofType:(const char*)type forPointer:(void*)ptr
{
    
}
- (void) setValue:value forInvocation:(NSInvocation*)inv atIndex:(int)n
{
    
}
- (void) setReturnValue:value forInvocation:(NSInvocation*)inv
{
    
}

- returnValueFromInvocation:(NSInvocation*)inv
{
    return nil;
}

+ (NSString*) stringForKind:(long)kind
{
    switch(kind){
        default:
            return nil;
        case tk_short:
            return @"short";
        case tk_long:
            return @"long";
        case tk_ushort:
            return @"unsigned short";
        case tk_ulong:
            return @"unsigned long";
        case tk_float:
            return @"float";
        case tk_double:
            return @"double";
        case tk_boolean:
            return @"boolean";
        case tk_char:
            return @"char";
        case tk_string:
            return @"string";
        case tk_longlong:
            return @"long long";
        case tk_ulonglong:
            return @"unsigned long long";
        case tk_longdouble:
            return @"double";
        case tk_wchar:
            return @"wchar";
        case tk_wstring:
            return @"wstring";
        case tk_fixed:
            return @"fixed";
        case tk_octet:
            return @"octet";
        case tk_any:
            return @"any";
    }
}

+ _definitionFromCoder:(id<CORBADecoder>)coder
{
    return nil;
}

+ _definitionFromCoder:(id<CORBADecoder>)coder tag:(long)tag;
{
    return [self _definitionFromCoder:coder];
}

- _initWithCoder:(id<CORBADecoder>)coder
{
    [self doesNotRecognizeSelector:_cmd];
    return self;
}

+ definitionFromCoder:(id<CORBADecoder>)coder
{
    long location = [coder location];
    long kind;
    long size;
    IDLDefinition* def = nil;
    NSMutableData* data = nil;
    IDLEncapsulation* stream;
    
    [coder decodeLong:&kind];
    switch(kind){
        case tk_null:
        case tk_void:
            return nil;
        case tk_short:
        case tk_long:
        case tk_ushort:
        case tk_ulong:
        case tk_float:
        case tk_double:
        case tk_boolean:
        case tk_char:
        case tk_longlong:
        case tk_ulonglong:
        case tk_longdouble:
        case tk_wchar:
        case tk_fixed:
        case tk_octet:
        case tk_any:
            def = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionNamed:[self stringForKind:kind]];
            break;
        case tk_string:
        case tk_wstring:
            def = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionNamed:[self stringForKind:kind]];
            [coder decodeLong:&size];
            break;

        case tk_abstract_interface:
        case tk_local_interface:
        case tk_objref:
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            // Cannot contain a recursive indirection
            def = [IDLInterface _definitionFromCoder:stream tag:kind];
            break;
        case tk_struct:
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            def = [IDLStruct _definitionFromCoder:stream];
            break;
        case tk_union:
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            def = [IDLUnion _definitionFromCoder:stream];
            break;
        case tk_enum:
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            // Cannot contain recursive indirection
            def = [IDLEnum _definitionFromCoder:stream];
            break;
        case tk_sequence:
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            def = [IDLSequence _definitionFromCoder:stream];
            break;
        case tk_array:
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            def = [IDLArray _definitionFromCoder:stream];
            break;
        case tk_alias:
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            def = [IDLTypedef _definitionFromCoder:stream];
            break;
        case tk_except:
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            def = [IDLException _definitionFromCoder:stream];
            break;
        case -1:{
            long offset;
            [coder decodeLong:&offset];
            location += sizeof(long) + offset;
            def = [coder definitionWithLocation:location];
            break;
        }
        case tk_native:{
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            def = [IDLNative _definitionFromCoder:stream];
            break;
        }
        case tk_value_box:
        case tk_value:{
            [coder decodeData:&data];
            stream = [IDLEncapsulation encapsulationWithData:data];
            [stream setParentStream:coder location:location + sizeof(long)];
            def = [IDLValueType _definitionFromCoder:stream tag:kind];
            break;
        }
        default:
            [NSException raise:CORBANotImplemented
                format:@"Unsupported tc_kind '%d'",kind];
    }
    return def;
}

// BI_DIR support. Sending BI_DIR service context only if object reference is present
- (BOOL) willSendReference
{
    return NO;
}

// internal classes that encode/decode themselves should call these methods
+ objectForValue:(void*)data ofKind:(TCKind)kind
{
	return nil;
}
+ (void) setValue:(void*)data fromObject:object ofKind:(TCKind)kind
{
}
@end
