/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLLink.h"
#import "IDLDefinition_Private.h"
#import "IDLArray.h"
#import "IDLSequence.h"

@implementation IDLLink

- initWithDefinition:(IDLDefinition*)def
{
    self = [super init];
    _link = [def retain];
    return self;
}

+ linkWithDefinition:(IDLDefinition*)def
{
    return [[[self allocWithZone:[IDLDefinition zone]]initWithDefinition:def]autorelease];
}

- (void) setLink:(IDLDefinition*)link
{
    if(link != _link){
        [_link release];
        _link = [link retain];
    }
}

- (IDLDefinition*) link
{
    return _link;
}

- (void) dealloc
{
    [_link release];
    [super dealloc];
}

- (id <IDLDefinition>)referenceType
{
    return [_link referenceType];
}

- (NSMethodSignature*) methodSignatureForSelector:(SEL)selector
{
    return [_link methodSignatureForSelector:selector];
}

- (void) forwardInvocation:(NSInvocation*)invocation
{
    [invocation invokeWithTarget:_link];
}

- valueForKey:(NSString*)key
{
	return [_link valueForKey:key];
}

- (NSString*) name
{
    return [_link name];
}

- (void) setNamePath:(NSString*)path
{
}

- sequenceClass
{
    return [IDLSequence class];
}

- arrayClass
{
    return [IDLArray class];
}

- (NSString*) description
{
    return [NSString stringWithFormat:@"&%@", _link];
}
- (id)replacementObjectForKeyedArchiver:(NSKeyedArchiver *)archiver
{
	return _link;
}
- (id)replacementObjectForCoder:(NSCoder *)aCoder
{
	return _link;
}

@end
