/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLIOPProfile.h"
#import "IDLProfileBody.h"
#import "IDLDefinitionScope.h"
#import "IDLEncapsulation.h"
#import "IDLGIOPVersion.h"
#import "IDLDefinition_Private.h"

@implementation IDLIOPProfile

- (void) encode
{
    IDLDefinition* versObj = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionForNamePath:@"IIOP/Version"];
    if(versObj){
        id version = [_profileBody iiopVersion];
        unsigned major = [version majorVersion];
        unsigned minor = [version minorVersion];
        NSString* path = [NSString stringWithFormat:@"IIOP/ProfileBody_%d_%d", major, minor];
        IDLDefinition* bodyObj = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionForNamePath:path];
        if(bodyObj){
            IDLEncapsulation* stream = [IDLEncapsulation encapsulation];
            [bodyObj encodeObject:_profileBody withCoder:stream];
            [_profile_data release];
            _profile_data = [[stream data]retain];
        }else
            [NSException raise:CORBADefinitionException
                        format:@"Cannot find definition for '%@'",path];
    }else
        [NSException raise:CORBADefinitionException
                    format:@"Cannot find definition for 'IIOP/Version'"];
}

- (void) _decode
{
    IDLDefinition* versObj = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionForNamePath:@"IIOP/Version"];
    if(versObj){
        IDLEncapsulation* stream = [IDLEncapsulation encapsulationWithData:(NSMutableData*)_profile_data];
        id obj = nil;
        [stream peepStruct:&obj usingObjectCoder:versObj];
        {
            unsigned major = [obj majorVersion];
            unsigned minor = [obj minorVersion];
            NSString* path = [NSString stringWithFormat:@"IIOP/ProfileBody_%d_%d", major, minor];
            IDLDefinition* bodyObj = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionForNamePath:path];
            if(bodyObj){
                [bodyObj decodeObject:&obj withCoder:stream];
                _profileBody = [obj retain];
            }else
                [NSException raise:CORBADefinitionException
                            format:@"Cannot find definition for '%@'",path];
        }
    }else
        [NSException raise:CORBADefinitionException
                    format:@"Cannot find definition for 'IIOP/Version'"];
}

- (void) decode
{
    if(_tag == TAG_INTERNET_IOP)
        [self _decode];
}

- initWithIIOPBody:(IDLProfileBody*)body
{
    self = [super init];
    _tag = TAG_INTERNET_IOP;
    _profileBody = [body retain];
	_local = YES;
//    [self encode];
    return self;
}

- initWithCorbaloc:(NSString*)corbaloc
{
    IDLProfileBody* body = [IDLProfileBody profileWithCorbaloc:corbaloc];
    return [self initWithIIOPBody:body];
}

- (void) dealloc
{
    [_profileBody release];
    [_profile_data release];
    [super dealloc];
}

+ iopProfileWithBody:(IDLProfileBody*)body
{
    return [[[self alloc]initWithIIOPBody:body]autorelease];
}

+ iopProfileWithCorbaloc:(NSString*)corbaloc;
{
    return [[[self alloc]initWithCorbaloc:corbaloc]autorelease];
}

- (unsigned long) tag
{
    return _tag;
}

- (void) setProfile_data:(NSData*)data
{
    if(_profile_data != data){
        [_profile_data release];
        _profile_data = [data retain];
        [self decode];
    }
}

- (NSData*) profile_data
{
	// return profile_data set by 'setProfile_data:' method as is
	// otherwise always encode it from profile body
    if(!_profile_data || _local) 
		[self encode];
    return _profile_data;
}

- (IDLProfileBody*) profileBody
{
    return _profileBody;
}

- (void) addComponent:(IDLTaggedComponent*)component
{
    [_profileBody addComponent:component];
//    [self encode];
}
- (IDLTaggedComponent*) componentWithTag:(unsigned)tag
{
	return [_profileBody componentWithTag:tag];
}

@end

#import "CORBAObject.h"

@implementation CORBAObject(TaggedComponents)

- (NSArray*) taggedComponents
{
	IDLIOPProfile* profile = [self profileWithTag:TAG_INTERNET_IOP];
	if(profile)
		return [[profile profileBody]components];
	else
		return nil;
}

- (void) addTaggedComponent:(IDLTaggedComponent*)component
{
	if([self isLocal]){
		IDLIOPProfile* profile = [self profileWithTag:TAG_INTERNET_IOP];
		if(profile)
			[profile addComponent:component];
		else
			[NSException raise:NSInternalInconsistencyException 
				format:@"cannot add tagged component - object has no profile"];
	}else
		[NSException raise:NSInternalInconsistencyException
			format:@"cannot add tagged component - object is not local"];
}

- (IDLTaggedComponent*) taggedComponentWithTag:(unsigned)tag
{
	IDLIOPProfile* profile = [self profileWithTag:TAG_INTERNET_IOP];
	if(profile)
		return [profile componentWithTag:tag];
	return nil;
}

@end

