/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>

@class VXConnection;

@interface VXConnectionObserver : NSObject
{
    VXConnection* _connection;
}

// Observer is created automatically when new connection is accepted, only if the observer class is set. 
// You should not create it yourself
// You don't need to retain the connection observers - they are retained and released automatically

// Returns observer for current request (in server)
+ observerForCurrentRequest;

// Return the observer for the given connection. You never should call this method directly
+ observerWithConnection:connection;

// Called just after creating the observer
- (void) connectionAccepted;

// Called when connection is destroyed.
// You should release the observer here if you have retained it
- (void) connectionDestroyed;

// Returns the observer class
+ observerClass;

// Sets the observer class. 
// The observers are not created unless the observer class is set.
+ (void) setObserverClass:newClass;

// Remote host inquiries
- (NSString*)hostName;
- (NSNumber*)port;
- (NSString*)address;

@end
